@@ -19,10 +19,11 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 from .ordered import OrderedProperty, OrderedModel
-from .subfield import CompositeString
+from .subfield import CompositeString, CompositeField
 import json
 import colander
 import deform
+import hashlib
 
 class Document(OrderedModel):
 
@@ -62,7 +63,10 @@ class Document(OrderedModel):
         properties = {}
         for prop in self:
             descriptor = self.__class__.__getattribute__(self.__class__, prop)
-            properties[prop] = descriptor._pystruct(self, getattr(self, prop, None))
+            try:
+                properties[prop] = descriptor._pystruct(self, getattr(self, prop))
+            except AttributeError:
+                pass
         if not 'TYPE' in properties:
             properties['TYPE'] = self.TYPE
 
@@ -74,7 +78,7 @@ class Document(OrderedModel):
             raise TypeError()
 
         isisdm_pystruct = dict((str(k), tuple(v) if isinstance(v, list) else v)
-            for k, v in pystruct.items())
+            for k, v in pystruct.items() if v is not None)
 
         return cls(**isisdm_pystruct)
 
@@ -93,7 +97,7 @@ class CheckedProperty(OrderedProperty):
         super(CheckedProperty, self).__init__()
         self.required = required
         self.validator = validator
-        self.choices = choices if choices else []
+        self.choices = choices if choices else ()
 
     def __set__(self, instance, value):
         if self.validator:
@@ -130,7 +134,15 @@ class TextProperty(CheckedProperty):
         return value
 
     def _colander_schema(self, instance, value):
-        return colander.SchemaNode(colander.String(), name=self.name)
+        kwargs = {'name':self.name}
+        if not self.required:
+            kwargs.update({'missing':None})
+        if self.choices:
+            choices_keys = [item[0] for item in self.choices]
+            kwargs.update({'widget':deform.widget.SelectWidget(values=self.choices),
+                           'validator':colander.OneOf(choices_keys)})
+        
+        return colander.SchemaNode(colander.String(), **kwargs)
 
 class BooleanProperty(CheckedProperty):
 
@@ -146,31 +158,35 @@ class BooleanProperty(CheckedProperty):
         return value
 
     def _colander_schema(self, instance, value):
-        return colander.SchemaNode(colander.Boolean(), name=self.name)
+        kwargs = {'name':self.name}
+        if not self.required:
+            kwargs.update({'missing':None})
+
+        return colander.SchemaNode(colander.Boolean(), **kwargs)
 
 class FileProperty(CheckedProperty):
 
     def __set__(self, instance, value):
         if not isinstance(value, dict):
             raise TypeError('%r must be a dictionary' % self.name)
-        
-        if 'fp' not in value:
-            raise TypeError('fp value must exists')
-        
+       
         if 'filename' not in value:
             try:
                 value['filename'] = value['fp'].name
             except AttributeError:
                 raise TypeError('%r must be a file' % self.name)
-
-
+        
         super(FileProperty, self).__set__(instance, value)
 
     def _pystruct(self, instance, value):
         '''
         python representation for this property
-        '''
-        value['fp'] = None        
+        '''        
+        if isinstance(value, dict):
+            serializable_value = {'uid':value['uid'],
+                                  'filename':value['filename'],}
+            return serializable_value
+
         return value
 
     def _colander_schema(self, instance, value):
@@ -178,9 +194,13 @@ class FileProperty(CheckedProperty):
             def preview_url(self, name):
                 return None
         tmpstore = MemoryTmpStore()
-        return colander.SchemaNode(deform.FileData(), 
-                                   widget=deform.widget.FileUploadWidget(tmpstore),
-                                   name=self.name)
+
+        kwargs = {'widget':deform.widget.FileUploadWidget(tmpstore),
+                  'name':self.name}
+        if not self.required:
+            kwargs.update({'missing':None})
+
+        return colander.SchemaNode(deform.FileData(), **kwargs)
 
 class MultiTextProperty(CheckedProperty):
 
@@ -196,16 +216,20 @@ class MultiTextProperty(CheckedProperty):
         return value
 
     def _colander_schema(self, instance, value):
-        schema = colander.SchemaNode(colander.Sequence(), name=self.name)
+        kwargs = {'name':self.name}
+        if not self.required:
+            kwargs.update({'missing':None})
+
+        schema = colander.SchemaNode(colander.Sequence(), **kwargs)
         schema.add(colander.SchemaNode(colander.String(), name=self.name))
 
         return schema
 
 
-class CompositeTextProperty(CheckedProperty):
+class IsisCompositeTextProperty(CheckedProperty):
 
     def __init__(self, subkeys=None, **kwargs):
-        super(CompositeTextProperty, self).__init__(**kwargs)
+        super(IsisCompositeTextProperty, self).__init__(**kwargs)
         self.subkeys = subkeys
 
     def __set__(self, instance, value):
@@ -213,7 +237,7 @@ class CompositeTextProperty(CheckedProperty):
             raise TypeError('%r value must be unicode or str instance' % self.name)
 
         composite_text = CompositeString(value, self.subkeys)
-        super(CompositeTextProperty, self).__set__(instance, composite_text)
+        super(IsisCompositeTextProperty, self).__set__(instance, composite_text)
 
     def _pystruct(self, instance, value):
         '''
@@ -230,18 +254,57 @@ class CompositeTextProperty(CheckedProperty):
                                    subfield,
                                    name=self.name)
 
-class MultiCompositeTextProperty(CheckedProperty):
+class CompositeTextProperty(CheckedProperty):
+
+    def __init__(self, subkeys, **kwargs):
+        super(CompositeTextProperty, self).__init__(**kwargs)
+        if not isinstance(subkeys,tuple) and not isinstance(subkeys, list):
+            raise TypeError('subkeys argument must be tuple or list')
+        self.subkeys = subkeys
+
+    def __set__(self, instance, value):
+        try:
+            value_as_dict = dict(value)
+        except ValueError:
+            raise TypeError('%r value must be a key-value structure' % self.name)
+
+        try:
+            value = CompositeField(value_as_dict, self.subkeys)
+        except TypeError:
+            raise TypeError('%r got an unexpected keyword' % self.name)
+
+        super(CompositeTextProperty, self).__set__(instance, value)
+
+    def _pystruct(self, instance, value):
+        '''
+        python representation for this property
+        '''
+        return value.items()
+
+    def _colander_schema(self, instance, value):
+        #option arg acts in each attribute
+        kwargs = {}
+        if not self.required:
+            kwargs.update({'missing':None})
+
+        subfield = colander.SchemaNode(colander.Mapping(), name=self.name)
+        for subkey in self.subkeys:
+            subfield.add(colander.SchemaNode(colander.String(), name=subkey, **kwargs))
+
+        return subfield
+
+class MultiIsisCompositeTextProperty(CheckedProperty):
 
     def __init__(self, subkeys=None, **kwargs):
-        super(MultiCompositeTextProperty, self).__init__(**kwargs)
+        super(MultiIsisCompositeTextProperty, self).__init__(**kwargs)
         self.subkeys = subkeys
 
     def __set__(self, instance, value):
         if not isinstance(value, tuple):
-            raise TypeError('MultiCompositeText value must be tuple')
+            raise TypeError('MultiIsisCompositeText value must be tuple')
 
         composite_texts = tuple(CompositeString(raw_composite_text, self.subkeys) for raw_composite_text in value)
-        super(MultiCompositeTextProperty, self).__set__(instance, composite_texts)
+        super(MultiIsisCompositeTextProperty, self).__set__(instance, composite_texts)
 
     def _pystruct(self, instance, value):
         '''
@@ -258,6 +321,46 @@ class MultiCompositeTextProperty(CheckedProperty):
                                    schema,
                                    name=self.name)
 
+class MultiCompositeTextProperty(CheckedProperty):
+
+    def __init__(self, subkeys, **kwargs):
+        super(MultiCompositeTextProperty, self).__init__(**kwargs)
+        if not isinstance(subkeys,tuple) and not isinstance(subkeys, list):
+            raise TypeError('subkeys argument must be tuple or list')
+        self.subkeys = subkeys
+
+    def __set__(self, instance, value):
+        if not isinstance(value, tuple) and not isinstance(value, list):
+            raise TypeError('%r value must be tuple or list')
+        
+        try:
+            composite_texts = tuple(CompositeField(dict(composite_text), self.subkeys) for composite_text in value)            
+        except ValueError:
+            raise TypeError('%r value must be a list or tuple of key-value structures' % self.name)
+
+        super(MultiCompositeTextProperty, self).__set__(instance, composite_texts)
+
+    def _pystruct(self, instance, value):
+        '''
+        python representation for this property
+        '''
+        return tuple(composite_text.items() for composite_text in value)
+
+    def _colander_schema(self, instance, value):
+        schema = colander.SchemaNode(colander.Mapping(), name=self.name)
+        
+        kwargs = {}
+        if not self.required:
+            kwargs.update({'missing':None})
+        
+        for subkey in self.subkeys:
+            schema.add(colander.SchemaNode(colander.String(), name=subkey, **kwargs))
+
+        return colander.SchemaNode(colander.Sequence(),
+                                   schema,
+                                   name=self.name,
+                                   **kwargs)
+
 class ReferenceProperty(CheckedProperty):
 
     def __set__(self, instance, value):
