@@ -27,12 +27,12 @@ Object-document mapper
     ...
     >>> class Article(Document):
     ...     title = TextProperty(required=True, validator=text_validator)
-    ...     authors = CompositeTextProperty(required=False, subkeys='fl')
+    ...     authors = IsisCompositeTextProperty(required=False, subkeys='fl')
     ...     publisher = TextProperty(required=True, validator=text_validator)
     ...
     >>> class Magazine(Document):
     ...     title = TextProperty(required=True, validator=text_validator)
-    ...     authors = MultiCompositeTextProperty(required=False, subkeys='fl')
+    ...     authors = MultiIsisCompositeTextProperty(required=False, subkeys='fl')
     ...     pages = TextProperty(validator=is_number)
     ...
 
@@ -44,7 +44,7 @@ Using ReferenceProperty::
     ...
     >>> class BookWithinCollection(Document):
     ...     title = TextProperty(required=True, validator=text_validator)
-    ...     authors = MultiCompositeTextProperty(required=False, subkeys='fl')
+    ...     authors = MultiIsisCompositeTextProperty(required=False, subkeys='fl')
     ...     collection = ReferenceProperty()
     ...
 
@@ -63,7 +63,7 @@ Using FileProperty::
 
     >>> class BookWithAttachment(Document):
     ...     title = TextProperty(required=True, validator=text_validator)
-    ...     authors = MultiCompositeTextProperty(required=False, subkeys='fl')
+    ...     authors = MultiIsisCompositeTextProperty(required=False, subkeys='fl')
     ...     cover = FileProperty()
     ...
 
@@ -178,6 +178,9 @@ Colander Schema Generation::
     >>> ' '.join('%s:%s' % (c.name, type(c.typ).__name__) for c in sc.children)
     'title:String authors:Sequence pages:String'
 
+    >>> sc.serialize({'title':'My home', 'authors':['Edward','John'], 'pages':'22'}) == {'authors': [u'Edward', u'John'], 'pages': u'22', 'title': u'My home'}
+    True
+
     >>> book_with_bool_schema = BookWithBoolean.get_schema()
     >>> ' '.join('%s:%s' % (c.name, type(c.typ).__name__) for c in book_with_bool_schema.children)
     'title:String is_published:Boolean'
@@ -187,10 +190,43 @@ Hidding an attribute
     >>> book_with_meta_schema = BookWithinMeta.get_schema()
     >>> ' '.join('%s:%s' % (c.name, type(c.typ).__name__) for c in book_with_meta_schema.children)
     'title:String authors:Sequence pages:String'
+
+New CompositeText test
+
+    >>> class OtherBook(Document):
+    ...     author = CompositeTextProperty(subkeys=['name','role'])
+    ...
+    
+    >>> other = OtherBook(author=[['name','Braz, Marcelo'],['role','writer']])
+    >>> other.author
+    CompositeTuple((('name', 'Braz, Marcelo'), ('role', 'writer')))
+    >>> print other.author['name']
+    Braz, Marcelo
+    >>> other.to_python()
+    {'TYPE': 'OtherBook', 'author': (('name', 'Braz, Marcelo'), ('role', 'writer'))}
+    
+    >>> other_schema = OtherBook.get_schema()
+    >>> other_schema.serialize({'author':{'name':'john', 'role':'writer'}}) == {'author': {'role': u'writer', 'name': u'john'}}
+    True
+
+MultiCompositeTextProperty test
+
+    >>> class MultiCompositeOtherBook(Document):
+    ...     author = MultiCompositeTextProperty(subkeys=['name','role'])
+
+    >>> multi_other = MultiCompositeOtherBook(author=[[['name','Braz, Marcelo'],['role','writer']], [['name','Foo, Tony'],['role','editor']]])
+    >>> print multi_other.author[0]['name']
+    Braz, Marcelo
+    >>> multi_other.to_python() == {'TYPE': 'MultiCompositeOtherBook', 'author': ((('name', 'Braz, Marcelo'), ('role', 'writer')), (('name', 'Foo, Tony'), ('role', 'editor')))}
+    True
+    >>> multi_other_schema = MultiCompositeOtherBook.get_schema()
+    >>> multi_other_schema.serialize({'author':({'name':'john', 'role':'writer'}, {'name':'foo', 'role':'editor'})}) == {'author': [{'role': u'writer', 'name': u'john'}, {'role': u'editor', 'name': u'foo'}]}
+    True
 """
 from isis.model import Document
 from isis.model import TextProperty, MultiTextProperty
-from isis.model import CompositeTextProperty, MultiCompositeTextProperty
+from isis.model import CompositeTextProperty, IsisCompositeTextProperty
+from isis.model import MultiCompositeTextProperty, MultiIsisCompositeTextProperty
 from isis.model import ReferenceProperty, FileProperty, BooleanProperty
 
 def test():
